@@ -197,17 +197,43 @@ Red flags in binary scan results:
 - High component count with low confidence → Needs deeper analysis
 
 TOOL SELECTION GUIDE:
-- scan_directory: Primary tool for analyzing projects/codebases. Use for comprehensive license inventory, optional package identification, and vulnerability assessment
-- scan_binary: Analyze compiled binaries, executables, and archives (APK, EXE, DLL, SO, JAR). Detects OSS components and licenses in binaries using BinarySniffer. Use for mobile apps, firmware, libraries, and binary distributions
-- check_package: Use when you have a specific package identifier (PURL, CPE, or file) to analyze. Checks both vulnerabilities and licenses for individual packages
-- validate_policy: Standalone license policy validation. Use when you already have license data and need to check compliance
-- validate_license_list: Quick validation of license list for distribution safety. Answers "Can I ship MIT+Apache to App Store?" without requiring filesystem
-- get_license_obligations: Get detailed compliance requirements for specific licenses. Answers "What must I do to comply?"
-- check_license_compatibility: Check if two licenses can be used together. Answers "Can I mix MIT with GPL?"
-- get_license_details: Get comprehensive information about a license including full text. Use for generating NOTICE files
-- analyze_commercial_risk: Specialized tool for commercial distribution risk assessment. Detects copyleft licenses and mixed licensing in data files
-- generate_mobile_legal_notice: Creates legal notices for mobile app distribution. Use after identifying licenses in dependencies
-- generate_sbom: Automatically calls scan_directory internally to generate Software Bill of Materials
+
+**For Package Archives (.jar, .whl, .rpm, .gem, .nupkg, .crate, etc.):**
+- check_package: RECOMMENDED for package archives. Intelligently uses upmex for metadata extraction + osslili for licenses. Fastest and most accurate for packages with structured metadata.
+
+**For Compiled Binaries (.so, .dll, .dylib, .exe, .bin, .apk, .ipa):**
+- scan_binary: Use for truly compiled binaries, firmware, and mobile apps. Uses BinarySniffer for signature-based component detection. Best for executables and native libraries.
+
+**For Source Code Directories:**
+- scan_directory: Primary tool for analyzing projects/codebases. Performs comprehensive license inventory with osslili, optional package identification, and vulnerability assessment.
+
+**Detailed Tool Descriptions:**
+- check_package: Intelligent package analyzer that automatically selects the best extraction method:
+  * For archives: Tries upmex first (fast, accurate metadata), falls back to osslili
+  * For PURLs: Direct package registry lookups
+  * Returns: Package metadata (name, version, PURL), licenses, optional vulnerabilities
+  * Use when: You have a package file (.jar, .whl, etc.) or PURL to analyze
+
+- scan_binary: Binary signature detection for compiled files using BinarySniffer:
+  * Detects OSS components embedded in binaries through signature matching
+  * Use for: Mobile apps (APK/IPA), executables (EXE), native libraries (SO/DLL/DYLIB), firmware
+  * Returns: Detected components, licenses, compatibility warnings, optional SBOM
+  * Note: Slower than check_package for archives; prefer check_package for .jar, .whl, etc.
+
+- scan_directory: Comprehensive source code analysis:
+  * License inventory via osslili (fast, thorough)
+  * Optional package identification via src2purl
+  * Optional vulnerability scanning (first 10 packages)
+  * Use for: Git repositories, source directories, projects with build files
+
+- validate_policy: Standalone license policy validation without filesystem access
+- validate_license_list: Quick distribution safety check (e.g., "Can I ship to App Store?")
+- get_license_obligations: Detailed compliance requirements for specific licenses
+- check_license_compatibility: Check if two licenses can be combined
+- get_license_details: Comprehensive license information including full text
+- analyze_commercial_risk: Commercial distribution risk assessment for copyleft detection
+- generate_mobile_legal_notice: Creates legal notices for mobile app compliance
+- generate_sbom: Generates Software Bill of Materials (calls scan_directory internally)
 
 PERFORMANCE CONSTRAINTS:
 1. Vulnerability scanning limited to first 10 packages to avoid timeouts
@@ -530,45 +556,137 @@ async def check_package(
     check_vulnerabilities: bool = True,
     check_licenses: bool = True
 ) -> Dict[str, Any]:
-    """Check a specific package."""
-    result = {}
+    """Check a specific package using intelligent tool selection.
+
+    This tool intelligently analyzes package files by:
+    1. For archives (.jar, .whl, .rpm, .gem, etc.): Use upmex for metadata extraction
+    2. If upmex fails or for non-archives: Fall back to osslili for license detection
+    3. For PURLs: Use package registry APIs when available
+
+    Args:
+        identifier: Package identifier (PURL like pkg:maven/com.google.gson/gson@2.10.1,
+                   file path to archive, or package file)
+        check_vulnerabilities: Whether to check for vulnerabilities (default: False for speed)
+        check_licenses: Whether to extract license information (default: True)
+
+    Returns:
+        Dictionary containing package metadata, licenses, and optionally vulnerabilities
+    """
+    result = {
+        "identifier": identifier,
+        "purl": None,
+        "package_info": {},
+        "licenses": [],
+        "extraction_method": None
+    }
 
     try:
         # Determine identifier type
         if identifier.startswith("pkg:"):
-            # It's a PURL
+            # It's already a PURL
+            result["purl"] = identifier
             purl = identifier
         elif identifier.startswith("cpe:"):
-            # It's a CPE
+            # It's a CPE - limited support
             purl = None
+            result["extraction_method"] = "cpe"
         else:
-            # Try to identify as a file
-            src2purl_result = _run_tool("src2purl", [identifier])
-            if src2purl_result.returncode == 0 and src2purl_result.stdout:
-                package_info = json.loads(src2purl_result.stdout)
-                purl = package_info.get("purl")
+            # It's a file path - use intelligent detection
+            file_path = Path(identifier)
+            if not file_path.exists():
+                return {"error": f"File not found: {identifier}"}
+
+            # Check if it's a package archive
+            archive_extensions = {'.jar', '.war', '.ear', '.whl', '.egg', '.tar.gz', '.tgz',
+                                '.gem', '.nupkg', '.rpm', '.deb', '.apk', '.crate', '.conda'}
+
+            is_archive = any(str(file_path).endswith(ext) for ext in archive_extensions)
+
+            if is_archive:
+                # Try upmex first for package archives
+                logger.info(f"Detected archive file, attempting upmex extraction: {identifier}")
+                try:
+                    upmex_result = _run_tool("upmex", ["extract", identifier], timeout=60)
+                    if upmex_result.returncode == 0 and upmex_result.stdout:
+                        logger.info(f"upmex raw stdout length: {len(upmex_result.stdout)}")
+                        if not upmex_result.stdout.strip():
+                            logger.warning("upmex returned empty output")
+                            purl = None
+                        else:
+                            package_data = json.loads(upmex_result.stdout)
+                            result["package_info"] = package_data.get("package", {})
+                            result["purl"] = package_data.get("package", {}).get("purl")
+                            result["extraction_method"] = "upmex"
+                            purl = result["purl"]
+                            logger.info(f"Successfully extracted package metadata with upmex: {purl}")
+                    else:
+                        logger.warning(f"upmex failed: returncode={upmex_result.returncode}, stderr={upmex_result.stderr}")
+                        purl = None
+                except json.JSONDecodeError as e:
+                    logger.error(f"Failed to parse upmex JSON output: {e}")
+                    logger.error(f"upmex stdout was: {upmex_result.stdout[:500] if upmex_result.stdout else 'None'}")
+                    purl = None
+                except Exception as e:
+                    logger.warning(f"upmex extraction error: {e}, falling back to osslili")
+                    purl = None
             else:
                 purl = None
 
-        # Check vulnerabilities
+        # Extract license information
+        if check_licenses:
+            if purl and result["extraction_method"] == "upmex":
+                # For upmex-extracted packages, also run osslili on the file for comprehensive license data
+                logger.info(f"Running osslili for comprehensive license detection on {identifier}")
+
+            # Run osslili on the file/archive
+            if not identifier.startswith("pkg:") and not identifier.startswith("cpe:"):
+                try:
+                    osslili_result = _run_tool("osslili", [identifier, "-f", "cyclonedx-json"], timeout=60)
+                    logger.info(f"osslili return code: {osslili_result.returncode}, stdout length: {len(osslili_result.stdout)}, stderr length: {len(osslili_result.stderr)}")
+
+                    if osslili_result.returncode == 0 and osslili_result.stdout:
+                        try:
+                            # osslili may output informational messages before JSON, find the JSON start
+                            json_start = osslili_result.stdout.find('{')
+                            if json_start > 0:
+                                json_output = osslili_result.stdout[json_start:]
+                            else:
+                                json_output = osslili_result.stdout
+
+                            license_data = json.loads(json_output)
+                            # Extract licenses from CycloneDX format
+                            if "components" in license_data:
+                                for component in license_data.get("components", []):
+                                    if "licenses" in component:
+                                        for lic in component["licenses"]:
+                                            if "license" in lic and "id" in lic["license"]:
+                                                result["licenses"].append(lic["license"]["id"])
+                            elif "licenses" in license_data:
+                                result["licenses"] = license_data["licenses"]
+                            result["extraction_method"] = result.get("extraction_method", "osslili") or "upmex+osslili"
+                            logger.info(f"License extraction successful: {len(result['licenses'])} licenses found")
+                        except json.JSONDecodeError as e:
+                            logger.warning(f"Failed to parse osslili output: {e}")
+                            logger.warning(f"osslili stdout (first 500 chars): {osslili_result.stdout[:500]}")
+                    else:
+                        logger.warning(f"osslili failed with return code {osslili_result.returncode}")
+                        if osslili_result.stderr:
+                            logger.warning(f"osslili stderr (first 1000 chars): {osslili_result.stderr[:1000]}")
+                except Exception as e:
+                    logger.warning(f"osslili execution failed: {e}, skipping license extraction")
+
+        # Check vulnerabilities if requested
         if check_vulnerabilities:
-            vulnq_args = [identifier, "--format", "json"]
-            vulnq_result = _run_tool("vulnq", vulnq_args)
-            if vulnq_result.returncode == 0 and vulnq_result.stdout:
-                vuln_data = json.loads(vulnq_result.stdout)
-                result["vulnerabilities"] = vuln_data
-
-        # Check licenses
-        if check_licenses and purl:
-            purl2notices_args = ["-i", purl, "-f", "json"]
-            notices_result = _run_tool("purl2notices", purl2notices_args)
-            if notices_result.returncode == 0 and notices_result.stdout:
-                notices = json.loads(notices_result.stdout)
-                result["licenses"] = notices.get("licenses", [])
-                result["copyright"] = notices.get("copyright", "")
-
-        result["identifier"] = identifier
-        result["purl"] = purl
+            if result["purl"]:
+                vulnq_result = _run_tool("vulnq", [result["purl"], "--format", "json"], timeout=30)
+                if vulnq_result.returncode == 0 and vulnq_result.stdout:
+                    vuln_data = json.loads(vulnq_result.stdout)
+                    result["vulnerabilities"] = vuln_data
+                else:
+                    result["vulnerabilities"] = []
+            else:
+                # No PURL available, cannot check vulnerabilities
+                result["vulnerabilities"] = []
 
     except Exception as e:
         logger.error(f"Error checking package: {e}")
@@ -1132,35 +1250,330 @@ async def generate_mobile_legal_notice(
 
 
 @mcp.tool()
-async def generate_sbom(
-    path: str,
-    format: str = "spdx",
-    output_file: Optional[str] = None
+async def generate_legal_notices(
+    purls: List[str],
+    output_format: str = "text",
+    output_file: Optional[str] = None,
+    include_license_text: bool = True
 ) -> Dict[str, Any]:
-    """Generate an SBOM."""
-    try:
-        # First, scan the directory
-        scan_result = await scan_directory(path, check_vulnerabilities=False)
+    """Generate comprehensive legal notices (attribution) for a list of packages using purl2notices.
 
-        sbom = {
-            "spdxVersion": "SPDX-2.3" if format == "spdx" else None,
-            "dataLicense": "CC0-1.0",
-            "name": Path(path).name,
-            "packages": scan_result.get("packages", []),
-            "licenses": scan_result.get("licenses", []),
-            "creationInfo": {
-                "created": "2025-01-05T00:00:00Z",
-                "creators": ["Tool: mcp-semclone-1.3.0"]
+    This tool generates legal compliance documentation including copyright notices,
+    license attributions, and full license texts for software packages. It's essential
+    for creating NOTICE files for distribution and ensuring legal compliance.
+
+    **Use this tool when:**
+    - You have a list of PURLs and need to generate attribution documentation
+    - Creating NOTICE files for software distribution
+    - Generating legal compliance documentation for products
+    - After scanning packages and need to document all licenses
+    - Preparing legal documentation for app store submissions
+
+    **When to use vs other tools:**
+    - Use this for COMPLETE legal notices with copyright and full license texts
+    - Use generate_mobile_legal_notice for simplified mobile app settings screens
+    - Use get_license_details for understanding individual license obligations
+    - This is typically the FINAL step after scanning/analyzing packages
+
+    Args:
+        purls: List of Package URLs (PURLs) to generate notices for
+        output_format: Output format - "text" (default), "html", "markdown"
+        output_file: Optional path to save the output file
+        include_license_text: If True, include full license texts (default: True)
+
+    Returns:
+        Dictionary containing:
+        - notices: The generated legal notices text
+        - packages_processed: Number of packages successfully processed
+        - packages_failed: Number of packages that failed processing
+        - output_file: Path to saved file (if output_file was specified)
+        - format: The output format used
+
+    Examples:
+        # Generate notices for analyzed packages
+        generate_legal_notices(
+            purls=["pkg:npm/express@4.0.0", "pkg:pypi/django@4.2.0"],
+            output_format="text"
+        )
+
+        # Generate HTML notices and save to file
+        generate_legal_notices(
+            purls=purl_list,
+            output_format="html",
+            output_file="/tmp/NOTICE.html"
+        )
+
+        # After batch scan, generate notices
+        scan_result = scan_directory("/path/to/project")
+        purls = [pkg["purl"] for pkg in scan_result["packages"]]
+        generate_legal_notices(purls=purls, output_file="NOTICE.txt")
+    """
+    import subprocess
+    import tempfile
+
+    try:
+        if not purls:
+            return {"error": "No PURLs provided"}
+
+        logger.info(f"Generating legal notices for {len(purls)} packages")
+
+        # Create temporary file with PURLs
+        with tempfile.NamedTemporaryFile(mode='w', suffix='.txt', delete=False) as purl_file:
+            for purl in purls:
+                purl_file.write(f"{purl}\n")
+            purl_file_path = purl_file.name
+
+        # Prepare purl2notices command
+        output_path = output_file or tempfile.mktemp(suffix=f'.{output_format}')
+
+        cmd = [
+            "purl2notices",
+            "-i", purl_file_path,
+            "-o", output_path,
+            "-f", output_format
+        ]
+
+        logger.info(f"Running purl2notices: {' '.join(cmd)}")
+
+        # Run purl2notices
+        result = subprocess.run(
+            cmd,
+            capture_output=True,
+            text=True,
+            timeout=300  # 5 minute timeout
+        )
+
+        # Clean up temp purl file
+        Path(purl_file_path).unlink(missing_ok=True)
+
+        if result.returncode != 0:
+            logger.error(f"purl2notices failed: {result.stderr}")
+            return {
+                "error": f"purl2notices failed: {result.stderr}",
+                "stdout": result.stdout,
+                "returncode": result.returncode
             }
+
+        # Read generated notices
+        if Path(output_path).exists():
+            with open(output_path, 'r') as f:
+                notices_content = f.read()
+
+            return {
+                "notices": notices_content,
+                "packages_processed": len(purls),
+                "packages_failed": 0,
+                "output_file": output_path,
+                "format": output_format,
+                "message": f"Successfully generated legal notices for {len(purls)} packages"
+            }
+        else:
+            return {
+                "error": "purl2notices completed but output file not found",
+                "stdout": result.stdout
+            }
+
+    except subprocess.TimeoutExpired:
+        logger.error("purl2notices timed out after 5 minutes")
+        return {"error": "Legal notices generation timed out after 5 minutes"}
+    except FileNotFoundError:
+        logger.error("purl2notices command not found")
+        return {
+            "error": "purl2notices not found. Install with: pip install purl2notices",
+            "install_command": "pip install purl2notices"
         }
+    except Exception as e:
+        logger.error(f"Error generating legal notices: {e}")
+        return {"error": str(e)}
+
+
+@mcp.tool()
+async def generate_sbom(
+    purls: Optional[List[str]] = None,
+    path: Optional[str] = None,
+    output_format: str = "cyclonedx-json",
+    output_file: Optional[str] = None,
+    include_licenses: bool = True
+) -> Dict[str, Any]:
+    """Generate a Software Bill of Materials (SBOM) from packages or directory scan.
+
+    This tool creates comprehensive SBOMs in industry-standard formats (CycloneDX, SPDX)
+    for software inventory, vulnerability tracking, and compliance documentation.
+
+    **Use this tool when:**
+    - You need to generate an SBOM for a project or package list
+    - Creating inventory documentation for compliance
+    - After analyzing packages and need structured output
+    - Preparing documentation for security audits
+    - Required by procurement or regulatory requirements
+
+    **Input modes:**
+    - Provide `purls` (list of Package URLs) for packages you've already identified
+    - Provide `path` to scan a directory and generate SBOM from discovered packages
+    - At least one of `purls` or `path` must be provided
+
+    Args:
+        purls: Optional list of Package URLs (PURLs) to include in SBOM
+        path: Optional directory path to scan for packages
+        output_format: SBOM format - "cyclonedx-json" (default), "cyclonedx-xml", "spdx-json", "spdx"
+        output_file: Optional path to save the SBOM file
+        include_licenses: If True, include license information (default: True)
+
+    Returns:
+        Dictionary containing:
+        - sbom: The generated SBOM structure
+        - format: The SBOM format used
+        - packages_count: Number of packages included
+        - output_file: Path to saved file (if output_file was specified)
+
+    Examples:
+        # Generate SBOM from PURLs (after batch analysis)
+        generate_sbom(
+            purls=["pkg:npm/express@4.0.0", "pkg:pypi/django@4.2.0"],
+            output_format="cyclonedx-json",
+            output_file="/tmp/sbom.json"
+        )
+
+        # Generate SBOM by scanning directory
+        generate_sbom(
+            path="/path/to/project",
+            output_format="spdx-json"
+        )
+
+        # After batch scan workflow
+        scan_result = check_package("package.jar")
+        generate_sbom(purls=[scan_result["purl"]], include_licenses=True)
+    """
+    import datetime
+
+    try:
+        if not purls and not path:
+            return {"error": "Either 'purls' or 'path' must be provided"}
+
+        packages_list = []
+        licenses_list = []
+        sbom_name = "project"
+
+        # If path provided, scan directory
+        if path:
+            logger.info(f"Generating SBOM by scanning directory: {path}")
+            scan_result = await scan_directory(path, check_vulnerabilities=False)
+            packages_list = scan_result.get("packages", [])
+            licenses_list = scan_result.get("licenses", [])
+            sbom_name = Path(path).name
+
+        # If PURLs provided, use them
+        elif purls:
+            logger.info(f"Generating SBOM from {len(purls)} PURLs")
+            for purl in purls:
+                # Parse PURL to extract package info
+                try:
+                    # Basic PURL parsing
+                    if purl.startswith("pkg:"):
+                        parts = purl[4:].split("/")
+                        ecosystem = parts[0]
+                        name_version = "/".join(parts[1:])
+
+                        if "@" in name_version:
+                            name, version = name_version.rsplit("@", 1)
+                        else:
+                            name = name_version
+                            version = "unknown"
+
+                        packages_list.append({
+                            "purl": purl,
+                            "name": name,
+                            "version": version,
+                            "ecosystem": ecosystem
+                        })
+                except Exception as e:
+                    logger.warning(f"Failed to parse PURL {purl}: {e}")
+
+        # Build SBOM based on format
+        if "cyclonedx" in output_format.lower():
+            sbom = {
+                "bomFormat": "CycloneDX",
+                "specVersion": "1.4",
+                "version": 1,
+                "metadata": {
+                    "timestamp": datetime.datetime.utcnow().isoformat() + "Z",
+                    "tools": [{
+                        "name": "mcp-semclone",
+                        "version": "1.3.4"
+                    }],
+                    "component": {
+                        "type": "application",
+                        "name": sbom_name
+                    }
+                },
+                "components": []
+            }
+
+            for pkg in packages_list:
+                component = {
+                    "type": "library",
+                    "name": pkg.get("name", "unknown"),
+                    "version": pkg.get("version", "unknown"),
+                    "purl": pkg.get("purl", "")
+                }
+
+                if include_licenses and "licenses" in pkg:
+                    component["licenses"] = pkg["licenses"]
+
+                sbom["components"].append(component)
+
+        else:  # SPDX format
+            sbom = {
+                "spdxVersion": "SPDX-2.3",
+                "dataLicense": "CC0-1.0",
+                "SPDXID": "SPDXRef-DOCUMENT",
+                "name": sbom_name,
+                "documentNamespace": f"https://semcl.one/sbom/{sbom_name}",
+                "creationInfo": {
+                    "created": datetime.datetime.utcnow().isoformat() + "Z",
+                    "creators": ["Tool: mcp-semclone-1.3.4"]
+                },
+                "packages": []
+            }
+
+            for pkg in packages_list:
+                spdx_pkg = {
+                    "SPDXID": f"SPDXRef-{pkg.get('name', 'unknown')}",
+                    "name": pkg.get("name", "unknown"),
+                    "versionInfo": pkg.get("version", "unknown"),
+                    "downloadLocation": "NOASSERTION"
+                }
+
+                if "purl" in pkg:
+                    spdx_pkg["externalRefs"] = [{
+                        "referenceCategory": "PACKAGE-MANAGER",
+                        "referenceType": "purl",
+                        "referenceLocator": pkg["purl"]
+                    }]
+
+                if include_licenses and "licenses" in pkg:
+                    spdx_pkg["licenseConcluded"] = pkg["licenses"][0] if pkg["licenses"] else "NOASSERTION"
+
+                sbom["packages"].append(spdx_pkg)
 
         # Save to file if requested
         if output_file:
             with open(output_file, "w") as f:
                 json.dump(sbom, f, indent=2)
-            return {"message": f"SBOM saved to {output_file}", "sbom": sbom}
+            logger.info(f"SBOM saved to {output_file}")
+            return {
+                "message": f"SBOM saved to {output_file}",
+                "sbom": sbom,
+                "format": output_format,
+                "packages_count": len(packages_list),
+                "output_file": output_file
+            }
 
-        return {"sbom": sbom}
+        return {
+            "sbom": sbom,
+            "format": output_format,
+            "packages_count": len(packages_list)
+        }
 
     except Exception as e:
         logger.error(f"Error generating SBOM: {e}")
